@@ -1,9 +1,10 @@
 
-""" Provides utilities to introspect #typing type hints. The goal is to assign the same semantics
-to all viable members of the #typing module independent of the Python version. """
+""" This module provides helper functions to introspect raw #typing type hints. It is used mostly internally.
+You should use #typeapi.of() and the datatypes defined in #typeapi.models to introspect type hints. """
 
 from __future__ import annotations
 import functools
+import inspect
 import sys
 import types
 import typing as t
@@ -68,6 +69,16 @@ class NewType(te.Protocol):
   __supertype__: t.Type
 
 
+class TypedDict(te.Protocol):
+  """ A protocol that describes #typing.TypedDict values (which are actually instances of the #typing._TypedDictMeta
+  metaclass). Use #is_typed_dict() to check if a hint is matches this protocol. """
+
+  __annotations__: t.Dict[str, t.Any]
+  __required_keys__: t.Set[str]
+  __optional_keys__: t.Set[str]
+  __total__: bool
+
+
 def is_generic(hint: t.Any) -> te.TypeGuard[t.Type[Generic]]:
   """
   Returns:
@@ -177,6 +188,23 @@ def is_new_type(hint: t.Any) -> te.TypeGuard[NewType]:
     return isinstance(hint, t.NewType)  # type: ignore[arg-type]
 
 
+def is_typed_dict(hint: t.Any) -> te.TypeGuard[TypedDict]:
+  """
+  Returns:
+    `True` if *hint* is a #typing.TypedDict.
+
+  !!! note
+
+    Typed dictionaries are actually just type objects. This means #typeapi.of() will represent them as
+    #typeapi.models.Type.
+  """
+
+  for m in (t, te):
+    if hasattr(m, '_TypedDictMeta') and isinstance(hint, m._TypedDictMeta):  # type: ignore[attr-defined]
+      return True
+  return False
+
+
 @functools.lru_cache()
 def get_special_generic_aliases() -> t.Dict[str, SpecialGenericAlias]:
   """ Returns a dictionary that contains all special generic aliases (like #typing.List and #typing.Mapping)
@@ -252,11 +280,37 @@ def type_repr(obj):
   return repr(obj)
 
 
-def get_type_hints(type_: t.Any) -> t.Dict[str, t.Any]:
+def get_annotations(type_: t.Any, include_bases: bool = False) -> t.Dict[str, t.Any]:
   """ Like #typing.get_type_hints(), but always includes extras. This is important when we want to inspect
-  #typing.Annotated hints (without extras the annotations are removed). """
+  #typing.Annotated hints (without extras the annotations are removed). In Python 3.10 and onwards, this is
+  an alias for #inspect.get_annotations() with `eval_str=True`.
 
-  if sys.version_info[:2] <= (3, 8):
-    return t.get_type_hints(type_)
+  If *include_bases* is set to `True`, annotations from base classes are taken into account as well. """
+
+
+  if sys.version_info[:2] <= (3, 9):
+    if sys.version_info[:2] <= (3, 8):
+      annotations = t.get_type_hints(type_)
+    else:
+      annotations = t.get_type_hints(type_, include_extras=True)
+    if not include_bases:
+      # To replicate the behaviour of #inspect.get_annotations(), which is to _not_ take into account
+      # the annotations of the base class, we discard all entries from the resulting dictionary that
+      # is not included in the types __annotations__.
+      local_annotations = getattr(type_, '__annotations__', {})
+      return {k: v for k, v in annotations.items() if k in local_annotations}
+    return annotations
+  elif isinstance(type_, type) and include_bases:
+    annotations = {}
+    for cls in type_.__mro__:
+      annotations.update({
+        k: v for k, v in inspect.get_annotations(cls, eval_str=True).items()
+        if k not in annotations
+      })
+    return annotations
   else:
-    return t.get_type_hints(type_, include_extras=True)
+    return inspect.get_annotations(type_, eval_str=True)
+
+
+# Backwards compatibility, remove in next minor version (minor because we're below 1.0.0)
+get_type_hints = get_annotations
